import sys
import simplejson as json
sys.path.insert(1, '/home/odoo/instance/odoo')
from odoo import release  # noqa: E402


print(json.dumps({
    'version': release.version,
    'major_version': release.major_version,
    'version_info': release.version_info}))



from distutils.util import strtobool
from shutil import copy
import argparse
import ast
import os
import sys
import fileinput

MANIFEST_FILES = ['__odoo__.py', '__openerp__.py', '__terp__.py', '__manifest__.py']
DEFAULT_FILE = '/home/odoo/.openerp_serverrc'


def is_module(path):
    """return False if the path doesn't contain an odoo module, and the full
    path to the module manifest otherwise"""

    if not os.path.isdir(path):
        return False
    files = os.listdir(path)
    filtered = [x for x in files if x in (MANIFEST_FILES + ['__init__.py'])]
    if len(filtered) == 2 and '__init__.py' in filtered:
        return os.path.join(
            path, next(x for x in filtered if x != '__init__.py'))
    else:
        return False


def is_installable_module(path):
    """return False if the path doesn't contain an installable odoo module,
    and the full path to the module manifest otherwise"""
    manifest_path = is_module(path)
    if manifest_path:
        manifest = ast.literal_eval(open(manifest_path).read())
        if manifest.get('installable', True):
            return manifest_path
    return False


def get_modules(paths):
    res = []
    for path in paths:
        if not os.path.isdir(path):
            continue
        module_paths = list(
            map(lambda module: os.path.join(path, module),
                os.listdir(path))
        )
        modules = [
            module for module in module_paths if
            is_installable_module(module)
        ]
        res.extend(modules)
    return res


def is_addons(path):
    return bool(get_modules([path]))


def get_addons(paths, ignore_deps=False, repositories_recursive=False):
    """Walks a path a returns all the addons inside sorted according to its
    dependencies.

    :param path: The path to walk
    :type path: str

    :param ignore_deps: Whether to take into account the repositories'
    dependencies or not
    :type ignore_deps: bool

    :returns: All the found addons
    :rtype: list
    """
    addons = {}
    for path in paths:
        if not os.path.isdir(path):
            continue
        if is_addons(path):
            addons[path] = []
        for base, dirs, files in os.walk(path):
            dir_paths = [os.path.join(base, directory) for directory in dirs]
            addons.update({
                directory: [] for directory in dir_paths if
                is_addons(directory)
            })
            if not repositories_recursive:
                break  # depth=1
    if ignore_deps:
        return addons
    for path in addons:
        addons[path] = get_dependencies(path)
    sorted_addons = get_sorted_addons_by_level(addons)
    return sorted_addons


def get_dependencies(path):
    """Gets the dependencies of an addon reading the `oca_dependencies.txt` file.

    :param path: The addon path
    :rtype: str

    :returns: The dependency list
    :rtype: list
    """
    deps = []
    dep_path = os.path.join(path, 'oca_dependencies.txt')
    try:
        with open(dep_path) as dep:
            for line in dep:
                line = line.strip()
                if not line or line.startswith('#'):
                    continue
                deps.append(line.split()[0])
    except IOError:
        return []
    return deps


def get_sorted_addons_by_level(addons):
    """Sorts a list of addons according to its level, and followed
    by its dependencies also sorted by level.

    It first sorts the addons using `sort_addons_by_level` and then
    it puts the dependencies after their parents. This to keep the addons
    with higher dependency level first and then its addons also sorted by
    level.

    :param addons: The addons with their dependencies.
    :type addons: dict

    :returns: The addons sorted by level
    :rtype: list

    Example::

        get_sorted_addons_by_level({
            "e": [],
            "r": ["x"],
            "c": [],
            "t": ["r", "c"],
            "x": ["e"],
        })

    Returns::

        ["t", "r", "c", "x", "e"]

    """
    sorted_addons = []
    addons_by_deps = sort_addons_by_level(addons)
    addons_by_deps = sorted(addons_by_deps.keys(),
                            key=lambda dep, addons=addons_by_deps: -addons.get(dep))
    for addon in addons_by_deps:
        if addon not in sorted_addons:
            sorted_addons.append(addon)
        for dep in addons.get(addon, []):
            dep_addon = os.path.join(os.path.dirname(addon), dep)
            if dep_addon not in sorted_addons:
                sorted_addons.append(dep_addon)
    return sorted_addons


def sort_addons_by_level(addons, _key=False, _res=False):
    """Reads a dict of addons with dependencies and assigns a "level" that represents
    how deep its dependencies go.

    :param addons: The addons with their dependencies.
    :type addons: dict

    :param _key: This is for recursion purposes. Do not pass when calling from the outside.
    :param _res: This is for recursion purposes. Do not pass when calling from the outside.

    :returns: The addons with the level
    :rtype: dict

    Example::

        sort_addons_by_level({
            "e": [],
            "r": ["x"],
            "c": [],
            "t": ["r", "c"],
            "x": ["e"],
        })

    Returns::

        {
            "t": 3,
            "r": 2,
            "x": 1,
            "c": 0,
            "e": 0,
        }

    In this example "t" depends on "r" that depends on "x" that depends on "e".
    There are three "levels" of dependencies below "t".
    """
    _res = _res or {}
    if not _key:
        for _key in addons:
            _res.update(sort_addons_by_level(addons, _key, _res))
        return _res
    if _res.get(_key, -1) > -1:
        return _res
    _res[_key] = 0
    for dep in addons.get(_key, []):
        _res.update(sort_addons_by_level(
            addons, os.path.join(os.path.dirname(_key), dep), _res))
    _res[_key] = (max([
        _res.get(os.path.join(os.path.dirname(_key), dep), -1)
        for dep in addons.get(_key, [])
    ] or [-1]) + 1)
    return _res


parser = argparse.ArgumentParser(
    description="Given a list  of paths, finds and returns a list of valid addons."
)
parser.add_argument(
    '-m', help="Only look for installable modules in the specified paths",
    action='store_true')
parser.add_argument(
    '-e', required=False,
    help='Comma separated list of modules to exclude'
)
parser.add_argument(
    '--ignore-deps', action='store_true',
    help=('Ignore the oca_dependencies of the repostories and generate the'
          ' addons path using the available modules')
)
parser.add_argument(
    '--odoo-addons', required=False, default='/home/odoo/instance/odoo/addons',
    help='Path where the odoo addons are located, defaults to /home/odoo/instance/odoo/addons'
)
parser.add_argument(
    '--repositories-recursive', action='store_true',
    default=strtobool(os.getenv('DEPLOYV_REPOSITORIES_RECURSIVE', "False")),
    help=('Looking for repositories into other repositories recursively addons paths (paths)')
)
parser.add_argument(
    '-f', '--rc-file', required=False, default=DEFAULT_FILE,
    help='Path to result openerp_serverrc file, defaults to {f}'.format(f=DEFAULT_FILE)
)
parser.add_argument(
    'paths', metavar='PATH',
    help="Comma separated list of paths where the addons are"
)


def main():
    args = parser.parse_args()
    list_modules = args.m
    exclude_modules = args.e.split(',') if args.e else []
    paths = args.paths.split(',')
    repositories_recursive = args.repositories_recursive
    if list_modules:
        addons_path = get_modules(paths)
    else:
        addons_path = get_addons(paths, args.ignore_deps, repositories_recursive)
    res = [module for module in addons_path if module not in exclude_modules]
    enterprise_path = [path for path in res if os.path.basename(path) == 'enterprise']
    if enterprise_path:
        res.remove(enterprise_path[0])
    addons_path = ",".join(enterprise_path + [args.odoo_addons] + res)

    filename = args.rc_file
    if filename != DEFAULT_FILE:
        copy(DEFAULT_FILE, filename)
    for line in fileinput.input(filename, inplace=True):
        if 'addons_path' in line:
            parts = line.split('=')
            new_str = '{field} = {addons}'.format(field=parts[0].strip(), addons=addons_path)
            print(new_str.replace('\n', ''))
        else:
            print(line.replace('\n', ''))


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_deployv-static
----------------------------------

Tests for `deployv-static` module.
"""

import deployv_static
import unittest
from os.path import isfile


class TestDeployv_static(unittest.TestCase):

    def setUp(self):
        pass

    def test_get_path(self):
        self.assertTrue(isfile(deployv_static.get_template_path('dev_instances.jinja')))

    def test_exception(self):
        with self.assertRaises(IOError):
            deployv_static.get_template_path('nonexistingtemplate.jinja')

    def tearDown(self):
        pass



import fnmatch
import pwd
import subprocess
import shlex
import sys
import os
from reqgen import reqgen


def linux_distribution():
    with open('/etc/os-release') as f:
        lines = f.readlines()
    for line in lines:
        key, value = line.split('=')
        key = key.strip()
        value = value.strip()
        if key == 'VERSION_CODENAME':
            if value:
                return value
        # This is a fix only for 14.04, we shouldn't need this newer versions
        if key == 'VERSION':
            if 'trusty' in value.lower():
                return 'trusty'
    raise Exception("Could not determine the distro version")


def read_lines(file_name):
    with open(file_name, 'r') as fileobj:
        lines = fileobj.readlines()
    return [line.strip() for line in lines if line.strip()]


def create_user():
    """ Creates the odoo user inside the container without password if it does not
     exists and then changes the password for the root user to odoo.
    """
    try:
        pwd.getpwnam('odoo')
    except KeyError:
        commands = [
            'adduser --home=/home/odoo --disabled-password --gecos "" --shell=/bin/bash odoo',
            'echo "root:odoo" | chpasswd',
            'chown odoo:odoo /home/odoo', ]
        for command in commands:
            subprocess.check_call(shlex.split(command))


def entry_point():
    """ Downloads the entrypoint used as command for any container created with this image
        and gives execution permissions to it.
    """
    commands = [
        ('wget -q -O-'
         ' https://raw.githubusercontent.com/Vauxoo/docker_entrypoint/master/installer.sh | sh'),
        ('wget -q -O /entry_point.py'
         ' https://raw.githubusercontent.com/Vauxoo/docker_entrypoint/master/entry_point.py'),
        'chmod +x /entry_point',
        'chmod +x /entry_point.py'
    ]
    for command in commands:
        subprocess.check_call(command, shell=True)


def search_apt_requirements(folder_name, file_name=None, recursive=False):
    """ Searches for all requirements.txt file recursively in the given path
    :param folder_name: the folder where you want to search
    :param file_name: filename to search
    :param recursive: Searches recursively and don't stop if the file is found
    :return: A list with all the files found, empty if none
    """
    res = list()
    if file_name is None:
        file_name = 'apt_requirements.txt'
    for base, directories, files in os.walk(folder_name):
        for file_name in fnmatch.filter(files, file_name):
            res.append(os.path.join(base, file_name))
            if not recursive:
                # Clean the directories so the walk doesn't go any further
                del directories[:]
    return res


def check_packages(package_list, dist):
    white_list = read_lines('/tmp/apt-package-{dist_name}'.format(dist_name=dist))
    for package_name in package_list:
        if package_name not in white_list:
            print('Could not install {} package, not whitelisted'.format(package_name))
            sys.exit(1)


def install_apt_requirements(apt_file, dist_name):
    """ Reads the apt_dependencies.txt file copied by the dockerfile in /tmp and parses it to
    install all the apt dependencies inside the container
    """
    deps = read_lines(apt_file)
    check_packages(deps, dist_name)
    for dep in deps:
        cmd = 'apt-get install {name} -yq'.format(name=dep.strip('\n'))
        subprocess.check_call(shlex.split(cmd))


def install_requirements():
    """ Gathers the requirements from all the requirements.txt files it can find in
        the folder where all the repos are with the help of reqgen,
        merges them into one single requirements.txt, and installs them.
    """
    home = os.environ.get("ODOO_HOME", '/home/odoo')
    instance = os.path.join(home, 'instance')
    full = os.path.join(home, 'full_requirements.txt')
    repo_path = os.path.join(instance, os.environ.get("MAIN_REPO_PATH"))
    exclude_path = os.path.join(repo_path, 'requirements_exclude.txt')
    if not os.path.exists(exclude_path):
        exclude_path = False
    reqgen.generate_merged_file(full, instance, exclude=exclude_path)
    cmd = 'pip install -r %s' % (full)
    subprocess.check_call(shlex.split(cmd))
    # Uninstall the requirements_exclude.txt if the path exists.
    if exclude_path:
        un_cmd = 'pip uninstall -r %s -yy' % (exclude_path)
        subprocess.check_call(shlex.split(un_cmd))

    # Generate test-requirements files without install
    # in order to be able to install it manually from "test_repo" option
    full = os.path.join(home, 'full_test-requirements.txt')
    reqgen.generate_merged_file(full, instance, requirements_name='test-requirements.txt')


def download_files(file_list, destination):
    for src in file_list:
        dest = os.path.join(destination, os.path.basename(src))
        command = 'wget -q -O {destination} {source}'.format(destination=dest, source=src)
        subprocess.check_call(shlex.split(command))


def apt_install():
    """ Execute the install from all apt_files found in the repos folder, but first
    updates the apt cache
    """
    dist_name = linux_distribution()
    apt_files = search_apt_requirements('/home/odoo/instance')
    apt_files.append('/tmp/apt_dependencies.txt')
    download_files([
        'https://git.vauxoo.com/vauxoo/apt_whitelist/raw/master/apt-package-{dist_name}'
        .format(dist_name=dist_name),
        'https://git.vauxoo.com/vauxoo/apt_whitelist/raw/master/apt-sources.json',
        ],
        '/tmp')
    subprocess.check_call(shlex.split('apt-get update'))
    for apt_file in apt_files:
        install_apt_requirements(apt_file, dist_name)


def create_paths():
    """ Creates the directories, and intermediate paths if needed, for some of the
        files that will be copied in the image.
    """
    paths = ['/home/odoo/.ssh', '/home/odoo/.local/share/Odoo', '/var/log/supervisor']
    for path in paths:
        try:
            os.makedirs(path)
        except OSError as error:
            if 'File exists' not in error.strerror:
                raise


def main():
    """ Main method in charge of following the logic required to set up the needed directories,
        install the python dependencies and set the addons path in the odoo config file.
    """
    commands = ['python /home/odoo/getaddons.py /home/odoo/instance/extra_addons', ]

    create_user()
    create_paths()
    entry_point()
    apt_install()
    install_requirements()
    for command in commands:
        subprocess.check_call(shlex.split(command))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = open('requirements.txt').readlines()
test_requirements = open('test-requirements.txt').readlines()

setup(
    name='deployv_static',
    version='0.0.46',
    description="DeployV static files: Dockerfiles, json, configs, templates",
    long_description=readme + '\n\n' + history,
    author="Vauxoo",
    author_email='info@vauxoo.com',
    url='https://github.com/Vauxoo/deployv-static',
    packages=[
        'deployv_static',
    ],
    package_dir={'deployv_static':
                 'deployv_static'},
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='deployv-static',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



# -*- coding: utf-8 -*-
from os import path


def get_template_path(template_name):
    template_path = path.join(path.dirname(__file__), 'templates', template_name)
    if path.isfile(template_path):
        return template_path
    raise IOError('No such file %s', template_name)
